@@ -1,7 +1,7 @@
 /* mpfr_add -- add two floating-point numbers
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
-Contributed by the Spaces project, INRIA Lorraine.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -25,6 +25,9 @@ MA 02110-1301, USA. */
 int
 mpfr_add (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 {
+  MPFR_LOG_FUNC (("b[%#R]=%R c[%#R]=%R rnd=%d", b, b, c, c, rnd_mode),
+                 ("a[%#R]=%R", a, a));
+
   if (MPFR_ARE_SINGULAR(b,c))
     {
       if (MPFR_IS_NAN(b) || MPFR_IS_NAN(c))
