@@ -1,6 +1,7 @@
 /* mpfr_asinh -- inverse hyperbolic sine
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -62,7 +63,8 @@ mpfr_asinh (mpfr_ptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
     }
 
   /* asinh(x) = x - x^3/6 + ... so the error is < 2^(3*EXP(x)-2) */
-  MPFR_FAST_COMPUTE_IF_SMALL_INPUT (y, x, -2*MPFR_GET_EXP (x)+2,0,rnd_mode,{});
+  MPFR_FAST_COMPUTE_IF_SMALL_INPUT (y, x, -2 * MPFR_GET_EXP (x), 2, 0,
+                                    rnd_mode, {});
 
   Ny = MPFR_PREC (y);   /* Precision of output variable */
 
@@ -91,12 +93,14 @@ mpfr_asinh (mpfr_ptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
       (neg ? mpfr_sub : mpfr_add) (t, t, x, GMP_RNDN); /* sqrt(x^2+1)+x */
       mpfr_log (t, t, GMP_RNDN);                       /* ln(sqrt(x^2+1)+x)*/
 
-      /* error estimate -- see algorithms.ps */
-      err = Nt - (MAX (3 - MPFR_GET_EXP (t), 0) + 1);
-
-      if (MPFR_LIKELY (MPFR_IS_ZERO (t)
-                       || MPFR_CAN_ROUND (t, err, Ny, rnd_mode)))
-        break;
+      if (MPFR_LIKELY (MPFR_IS_PURE_FP (t)))
+        {
+          /* error estimate -- see algorithms.tex */
+          err = Nt - (MAX (4 - MPFR_GET_EXP (t), 0) + 1);
+          if (MPFR_LIKELY (MPFR_IS_ZERO (t)
+                           || MPFR_CAN_ROUND (t, err, Ny, rnd_mode)))
+            break;
+        }
 
       /* actualisation of the precision */
       MPFR_ZIV_NEXT (loop, Nt);
