@@ -1,6 +1,7 @@
 /* mpfr_atan2 -- arc-tan 2 of a floating-point number
 
-Copyright 2005, 2006 Free Software Foundation, Inc.
+Copyright 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library, and was contributed by Mathieu Dutour.
 
@@ -152,6 +153,12 @@ mpfr_atan2 (mpfr_ptr dest, mpfr_srcptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
       else
         goto set_zero;
     }
+
+  /* When x=1, atan2(y,x) = atan(y). FIXME: more generally, if x is a power
+     of two, we could call directly atan(y/x) since y/x is exact. */
+  if (mpfr_cmp_ui (x, 1) == 0)
+    return mpfr_atan (dest, y, rnd_mode);
+
   MPFR_SAVE_EXPO_MARK (expo);
 
   /* Set up initial prec */
@@ -163,10 +170,51 @@ mpfr_atan2 (mpfr_ptr dest, mpfr_srcptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
     /* use atan2(y,x) = atan(y/x) */
     for (;;)
       {
-        mpfr_div (tmp, y, x, GMP_RNDN);   /* Error <= ulp (tmp) */
+        int div_inex;
+        MPFR_BLOCK_DECL (flags);
+
+        MPFR_BLOCK (flags, div_inex = mpfr_div (tmp, y, x, GMP_RNDN));
+        if (div_inex == 0)
+          {
+            /* Result is exact. */
+            inexact = mpfr_atan (dest, tmp, rnd_mode);
+            goto end;
+          }
+
+        /* Error <= ulp (tmp) except in case of underflow or overflow. */
+
+        /* If the division underflowed, since |atan(z)/z| < 1, we have
+           an underflow. */
+        if (MPFR_UNDERFLOW (flags))
+          {
+            int sign;
+
+            /* In the case GMP_RNDN with 2^(emin-2) < |y/x| < 2^(emin-1):
+               The smallest significand value S > 1 of |y/x| is:
+                 * 1 / (1 - 2^(-px))                        if py <= px,
+                 * (1 - 2^(-px) + 2^(-py)) / (1 - 2^(-px))  if py >= px.
+               Therefore S - 1 > 2^(-pz), where pz = max(px,py). We have:
+               atan(|y/x|) > atan(z), where z = 2^(emin-2) * (1 + 2^(-pz)).
+                           > z - z^3 / 3.
+                           > 2^(emin-2) * (1 + 2^(-pz) - 2^(2 emin - 5))
+               Assuming pz <= -2 emin + 5, we can round away from zero
+               (this is what mpfr_underflow always does on GMP_RNDN).
+               In the case GMP_RNDN with |y/x| <= 2^(emin-2), we round
+               towards zero, as |atan(z)/z| < 1. */
+            MPFR_ASSERTN (MPFR_PREC_MAX <=
+                          2 * (mpfr_uexp_t) - MPFR_EMIN_MIN + 5);
+            if (rnd_mode == GMP_RNDN && MPFR_IS_ZERO (tmp))
+              rnd_mode = GMP_RNDZ;
+            sign = MPFR_SIGN (tmp);
+            mpfr_clear (tmp);
+            MPFR_SAVE_EXPO_FREE (expo);
+            return mpfr_underflow (dest, rnd_mode, sign);
+          }
+
         mpfr_atan (tmp, tmp, GMP_RNDN);   /* Error <= 2*ulp (tmp) since
                                              abs(D(arctan)) <= 1 */
-        /*FIXME: Error <= ulp(tmp) ? */
+        /* TODO: check that the error bound is correct in case of overflow. */
+        /* FIXME: Error <= ulp(tmp) ? */
         if (MPFR_LIKELY (MPFR_CAN_ROUND (tmp, prec - 2, MPFR_PREC (dest),
                                          rnd_mode)))
           break;
@@ -180,11 +228,13 @@ mpfr_atan2 (mpfr_ptr dest, mpfr_srcptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
       for (;;)
         {
           mpfr_div (tmp, y, x, GMP_RNDN);   /* Error <= ulp (tmp) */
+          /* If tmp is 0, we have |y/x| <= 2^(-emin-2), thus
+             atan|y/x| < 2^(-emin-2). */
           MPFR_SET_POS (tmp);               /* no error */
           mpfr_atan (tmp, tmp, GMP_RNDN);   /* Error <= 2*ulp (tmp) since
                                                abs(D(arctan)) <= 1 */
           mpfr_const_pi (pi, GMP_RNDN);     /* Error <= ulp(pi) /2 */
-          e = MPFR_GET_EXP (tmp);
+          e = MPFR_NOTZERO(tmp) ? MPFR_GET_EXP (tmp) : __gmpfr_emin - 1;
           mpfr_sub (tmp, pi, tmp, GMP_RNDN);          /* see above */
           if (MPFR_IS_NEG (y))
             MPFR_CHANGE_SIGN (tmp);
@@ -202,9 +252,10 @@ mpfr_atan2 (mpfr_ptr dest, mpfr_srcptr y, mpfr_srcptr x, mp_rnd_t rnd_mode)
         }
       mpfr_clear (pi);
     }
-  MPFR_ZIV_FREE (loop);
-
   inexact = mpfr_set (dest, tmp, rnd_mode);
+
+ end:
+  MPFR_ZIV_FREE (loop);
   mpfr_clear (tmp);
   MPFR_SAVE_EXPO_FREE (expo);
   return mpfr_check_range (dest, inexact, rnd_mode);
