@@ -1,6 +1,7 @@
 /* mpfr_const_pi -- compute Pi
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -70,18 +71,18 @@ mpfr_const_pi_internal (mpfr_ptr x, mp_rnd_t rnd_mode)
       {
         /* invariant: 1/2 <= B <= A <= a < 1 */
         mpfr_add (S, A, B, GMP_RNDN); /* 1 <= S <= 2 */
-        mpfr_div_2exp (S, S, 2, GMP_RNDN); /* exact, 1/4 <= S <= 1/2 */
+        mpfr_div_2ui (S, S, 2, GMP_RNDN); /* exact, 1/4 <= S <= 1/2 */
         mpfr_sqrt (b, B, GMP_RNDN); /* 1/2 <= b <= 1 */
         mpfr_add (ap, a, b, GMP_RNDN); /* 1 <= ap <= 2 */
-        mpfr_div_2exp (ap, ap, 1, GMP_RNDN); /* exact, 1/2 <= ap <= 1 */
+        mpfr_div_2ui (ap, ap, 1, GMP_RNDN); /* exact, 1/2 <= ap <= 1 */
         mpfr_mul (Ap, ap, ap, GMP_RNDN); /* 1/4 <= Ap <= 1 */
         mpfr_sub (Bp, Ap, S, GMP_RNDN); /* -1/4 <= Bp <= 3/4 */
-        mpfr_mul_2exp (Bp, Bp, 1, GMP_RNDN); /* -1/2 <= Bp <= 3/2 */
+        mpfr_mul_2ui (Bp, Bp, 1, GMP_RNDN); /* -1/2 <= Bp <= 3/2 */
         mpfr_sub (S, Ap, Bp, GMP_RNDN);
         MPFR_ASSERTN (mpfr_cmp_ui (S, 1) < 0);
         cancel = mpfr_cmp_ui (S, 0) ? (mpfr_uexp_t) -mpfr_get_exp(S) : p;
         /* MPFR_ASSERTN (cancel >= px || cancel >= 9 * (1 << k) - 4); */
-        mpfr_mul_2exp (S, S, k, GMP_RNDN);
+        mpfr_mul_2ui (S, S, k, GMP_RNDN);
         mpfr_sub (D, D, S, GMP_RNDN);
         /* stop when |A_k - B_k| <= 2^(k-p) i.e. cancel >= p-k */
         if (cancel + k >= p)
