@@ -1,7 +1,7 @@
 /* mpfr_dump -- Dump a float to stdout.
 
-Copyright 1999, 2001, 2004, 2006 Free Software Foundation, Inc.
-(Copied from the GNU MP Library.)
+Copyright 1999, 2001, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -20,7 +20,6 @@ along with the MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
-#include <stdio.h>
 #include <string.h>
 
 #include "mpfr-impl.h"
