@@ -1,6 +1,7 @@
 /* mpfr_free_cache - Free the cache used by MPFR for internal consts.
 
-Copyright 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -21,6 +22,25 @@ MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
 
+#if 0
+static void
+free_l2b (void)
+{
+  int i, b;
+
+  for (b = 2; b <= BASE_MAX; b++)
+    for (i = 0; i < 2; i++)
+      {
+        mpfr_ptr p = __gmpfr_l2b[b-2][i];
+        if (p != NULL)
+          {
+            mpfr_clear (p);
+            (*__gmp_free_func) (p, sizeof (mpfr_t));
+          }
+      }
+}
+#endif
+
 void
 mpfr_free_cache (void)
 {
@@ -28,4 +48,5 @@ mpfr_free_cache (void)
   mpfr_clear_cache (__gmpfr_cache_const_log2);
   mpfr_clear_cache (__gmpfr_cache_const_euler);
   mpfr_clear_cache (__gmpfr_cache_const_catalan);
+  /* free_l2b (); */
 }
