@@ -1,6 +1,7 @@
 /* mpfr_gamma -- gamma function
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -138,6 +139,57 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
         }
     }
 
+  /* Check for tiny arguments, where gamma(x) ~ 1/x - euler + ....
+     We know from "Bound on Runs of Zeros and Ones for Algebraic Functions",
+     Proceedings of Arith15, T. Lang and J.-M. Muller, 2001, that the maximal
+     number of consecutive zeroes or ones after the round bit is n-1 for an
+     input of n bits. But we need a more precise lower bound. Assume x has
+     n bits, and 1/x is near a floating-point number y of n+1 bits. We can
+     write x = X*2^e, y = Y/2^f with X, Y integers of n and n+1 bits.
+     Thus X*Y^2^(e-f) is near from 1, i.e., X*Y is near from 2^(f-e).
+     Two cases can happen:
+     (i) either X*Y is exactly 2^(f-e), but this can happen only if X and Y
+         are themselves powers of two, i.e., x is a power of two;
+     (ii) or X*Y is at distance at least one from 2^(f-e), thus
+          |xy-1| >= 2^(e-f), or |y-1/x| >= 2^(e-f)/x = 2^(-f)/X >= 2^(-f-n).
+          Since ufp(y) = 2^(n-f) [ufp = unit in first place], this means
+          that the distance |y-1/x| >= 2^(-2n) ufp(y).
+          Now assuming |gamma(x)-1/x| <= 1, which is true for x <= 1,
+          if 2^(-2n) ufp(y) >= 2, the error is at most 2^(-2n-1) ufp(y),
+          and round(1/x) with precision >= 2n+2 gives the correct result.
+          If x < 2^E, then y > 2^(-E), thus ufp(y) > 2^(-E-1).
+          A sufficient condition is thus EXP(x) + 2 <= -2 MAX(PREC(x),PREC(Y)).
+  */
+  if (MPFR_EXP(x) + 2 <= -2 * (mp_exp_t) MAX(MPFR_PREC(x), MPFR_PREC(gamma)))
+    {
+      int positive = MPFR_IS_POS (x);
+      inex = mpfr_ui_div (gamma, 1, x, rnd_mode);
+      if (inex == 0) /* x is a power of two */
+        {
+          if (positive)
+            {
+              if (rnd_mode == GMP_RNDU || rnd_mode == GMP_RNDN)
+                inex = 1;
+              else /* round to zero or to -Inf */
+                {
+                  mpfr_nextbelow (gamma); /* 2^k - epsilon */
+                  inex = -1;
+                }
+            }
+          else /* negative */
+            {
+              if (rnd_mode == GMP_RNDU || rnd_mode == GMP_RNDZ)
+                {
+                  mpfr_nextabove (gamma); /* -2^k + epsilon */
+                  inex = 1;
+                }
+              else /* round to nearest and to -Inf */
+                inex = -1;
+            }
+        }
+      return inex;
+    }
+
   is_integer = mpfr_integer_p (x);
   /* gamma(x) for x a negative integer gives NaN */
   if (is_integer && MPFR_IS_NEG(x))
@@ -163,8 +215,14 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
       unsigned long int u;
       mp_prec_t p = MPFR_PREC(gamma);
       u = mpfr_get_ui (x, GMP_RNDN);
-      if (bits_fac (u - 1) <= p)
+      if (u < 44787929UL && bits_fac (u - 1) <= p + (rnd_mode == GMP_RNDN))
+        /* bits_fac: lower bound on the number of bits of m,
+           where gamma(x) = (u-1)! = m*2^e with m odd. */
         return mpfr_fac_ui (gamma, u - 1, rnd_mode);
+      /* if bits_fac(...) > p (resp. p+1 for rounding to nearest),
+         then gamma(x) cannot be exact in precision p (resp. p+1).
+         FIXME: remove the test u < 44787929UL after changing bits_fac
+         to return a mpz_t or mpfr_t. */
     }
 
   /* check for overflow: according to (6.1.37) in Abramowitz & Stegun,
@@ -172,25 +230,25 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
               >= 2 * (x/e)^x / x for x >= 1 */
   if (compared > 0)
     {
-      int overflow;
       mpfr_t yp;
+      MPFR_BLOCK_DECL (flags);
 
-      mpfr_clear_overflow ();
+      /* 1/e rounded down to 53 bits */
+#define EXPM1_STR "0.010111100010110101011000110110001011001110111100111"
       mpfr_init2 (xp, 53);
       mpfr_init2 (yp, 53);
-      mpfr_set_d (xp, EXPM1, GMP_RNDZ); /* 1/e rounded down */
+      mpfr_set_str_binary (xp, EXPM1_STR);
       mpfr_mul (xp, x, xp, GMP_RNDZ);
       mpfr_sub_ui (yp, x, 2, GMP_RNDZ);
       mpfr_pow (xp, xp, yp, GMP_RNDZ); /* (x/e)^(x-2) */
-      mpfr_set_d (yp, EXPM1, GMP_RNDZ);
+      mpfr_set_str_binary (yp, EXPM1_STR);
       mpfr_mul (xp, xp, yp, GMP_RNDZ); /* x^(x-2) / e^(x-1) */
       mpfr_mul (xp, xp, yp, GMP_RNDZ); /* x^(x-2) / e^x */
-      mpfr_mul (xp, xp, x, GMP_RNDZ); /* x^(x-1) / e^x */
-      mpfr_mul_2exp (xp, xp, 1, GMP_RNDZ);
-      overflow = mpfr_overflow_p ();
+      mpfr_mul (xp, xp, x, GMP_RNDZ); /* lower bound on x^(x-1) / e^x */
+      MPFR_BLOCK (flags, mpfr_mul_2ui (xp, xp, 1, GMP_RNDZ));
       mpfr_clear (xp);
       mpfr_clear (yp);
-      return (overflow) ? mpfr_overflow (gamma, rnd_mode, 1)
+      return MPFR_OVERFLOW (flags) ? mpfr_overflow (gamma, rnd_mode, 1)
         : mpfr_gamma_aux (gamma, x, rnd_mode);
     }
 
@@ -235,12 +293,12 @@ mpfr_gamma (mpfr_ptr gamma, mpfr_srcptr x, mp_rnd_t rnd_mode)
       mpfr_const_pi (tmp2, GMP_RNDN);
       mpfr_mul (tmp2, tmp2, tmp, GMP_RNDN); /* Pi*(2-x) */
       mpfr_sin (tmp, tmp2, GMP_RNDN); /* sin(Pi*(2-x)) */
+      sgn = mpfr_sgn (tmp);
       mpfr_abs (tmp, tmp, GMP_RNDN);
       mpfr_mul_ui (tmp2, tmp2, 3, GMP_RNDU); /* 3Pi(2-x) */
       mpfr_add_ui (tmp2, tmp2, 1, GMP_RNDU); /* 3Pi(2-x)+1 */
-      mpfr_div_2exp (tmp2, tmp2, mpfr_get_prec (tmp), GMP_RNDU);
+      mpfr_div_2ui (tmp2, tmp2, mpfr_get_prec (tmp), GMP_RNDU);
       /* if tmp2<|tmp|, we get a lower bound */
-      sgn = mpfr_sgn (tmp);
       if (mpfr_cmp (tmp2, tmp) < 0)
         {
           mpfr_sub (tmp, tmp, tmp2, GMP_RNDZ); /* low bnd on |sin(Pi*(2-x))| */
