@@ -1,6 +1,7 @@
 /* __gmpfr_int_ceil_log2 -- Integer ceil of log2(x)
 
-Copyright 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -25,11 +26,17 @@ MA 02110-1301, USA. */
 int
 __gmpfr_int_ceil_log2 (unsigned long n)
 {
-  int b;
-  mp_limb_t limb = n;
+  if (MPFR_UNLIKELY (n == 1))
+    return 0;
+  else
+    {
+      int b;
+      mp_limb_t limb;
 
-  MPFR_ASSERTN (limb == n);
-
-  count_leading_zeros (b, limb);
-  return BITS_PER_MP_LIMB-b;
+      MPFR_ASSERTN (n > 1);
+      limb = n - 1;
+      MPFR_ASSERTN (limb == n - 1);
+      count_leading_zeros (b, limb);
+      return BITS_PER_MP_LIMB - b;
+    }
 }
