@@ -1,7 +1,8 @@
 /* mpfr_mul_ui -- multiply a floating-point number by a machine integer
    mpfr_mul_si -- multiply a floating-point number by a machine integer
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -72,7 +73,7 @@ mpfr_mul_ui (mpfr_ptr y, mpfr_srcptr x, unsigned long int u, mp_rnd_t rnd_mode)
         return mpfr_set (y, x, rnd_mode);
     }
   else if (MPFR_UNLIKELY (IS_POW2 (u)))
-    return mpfr_mul_2si (y, x, MPFR_INT_CEIL_LOG2 (u)-1, rnd_mode);
+    return mpfr_mul_2si (y, x, MPFR_INT_CEIL_LOG2 (u), rnd_mode);
 
   yp = MPFR_MANT (y);
   yn = MPFR_LIMB_SIZE (y);
