@@ -1,7 +1,7 @@
 /* mpfr_sgn -- Sign of a floating point number.
 
-Copyright 2003, 2004 Free Software Foundation.
-Contributed by the Spaces project, INRIA Lorraine.
+Copyright 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -25,5 +25,16 @@ MA 02110-1301, USA. */
 int
 (mpfr_sgn) (mpfr_srcptr a)
 {
-  return MPFR_UNLIKELY ( MPFR_IS_ZERO (a) ) ? 0 : MPFR_INT_SIGN (a);
+  if (MPFR_UNLIKELY (MPFR_IS_SINGULAR (a)))
+    {
+      if (MPFR_LIKELY (MPFR_IS_ZERO (a)))
+        return 0;
+      if (MPFR_UNLIKELY (MPFR_IS_NAN (a)))
+        {
+          MPFR_SET_ERANGE ();
+          return 0;
+        }
+      /* Remains infinity, handled by the return below. */
+    }
+  return MPFR_INT_SIGN (a);
 }
