@@ -1,7 +1,7 @@
 /* Test file for mpfr_atanh.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
-Adapted from tatan.c.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -179,6 +179,8 @@ main (int argc, char *argv[])
 
   test_generic (2, 100, 25);
 
+  data_check ("data/atanh", mpfr_atanh, "mpfr_atanh");
+
   tests_end_mpfr ();
   return 0;
 }
