@@ -1,6 +1,7 @@
 /* Test file for mpfr_cos.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -52,7 +53,7 @@ check53 (const char *xs, const char *cos_xs, mp_rnd_t rnd_mode)
 {
   mpfr_t xx, c;
 
-  mpfr_inits2 (53, xx, c, NULL);
+  mpfr_inits2 (53, xx, c, (void *) 0);
   mpfr_set_str1 (xx, xs); /* should be exact */
   test_cos (c, xx, rnd_mode);
   if (mpfr_cmp_str1 (c, cos_xs))
@@ -64,10 +65,11 @@ check53 (const char *xs, const char *cos_xs, mp_rnd_t rnd_mode)
       printf(", expected %s\n", cos_xs);
       exit (1);
     }
-  mpfr_clears (xx, c, NULL);
+  mpfr_clears (xx, c, (void *) 0);
 }
 
 #define TEST_FUNCTION test_cos
+#define REDUCE_EMAX 262143 /* otherwise arg. reduction is too expensive */
 #include "tgeneric.c"
 
 static void
@@ -139,6 +141,10 @@ static void
 special_overflow (void)
 {
   mpfr_t x, y;
+  mp_exp_t emin, emax;
+
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
 
   mpfr_init2 (x, 24);
   mpfr_init2 (y, 73);
@@ -148,17 +154,96 @@ special_overflow (void)
   set_emax (128);
   mpfr_set_str_binary (x, "0.111101010110110011101101E6");
   test_cos (y, x, GMP_RNDZ);
-  set_emin (MPFR_EMIN_MIN);
-  set_emax (MPFR_EMAX_MAX);
+  set_emin (emin);
+  set_emax (emax);
 
   mpfr_clear (x);
   mpfr_clear (y);
 }
 
+static void
+overflowed_cos0 (void)
+{
+  mpfr_t x, y;
+  int emax, i, inex, rnd, err = 0;
+  mp_exp_t old_emax;
+
+  old_emax = mpfr_get_emax ();
+
+  mpfr_init2 (x, 8);
+  mpfr_init2 (y, 8);
+
+  for (emax = -1; emax <= 0; emax++)
+    {
+      mpfr_set_ui_2exp (y, 1, emax, GMP_RNDN);
+      mpfr_nextbelow (y);
+      set_emax (emax);  /* 1 is not representable. */
+      /* and if emax < 0, 1 - eps is not representable either. */
+      for (i = -1; i <= 1; i++)
+        RND_LOOP (rnd)
+        {
+          mpfr_set_si_2exp (x, i, -512 * ABS (i), GMP_RNDN);
+          mpfr_clear_flags ();
+          inex = mpfr_cos (x, x, rnd);
+          if ((i == 0 || emax < 0 || rnd == GMP_RNDN || rnd == GMP_RNDU) &&
+              ! mpfr_overflow_p ())
+            {
+              printf ("Error in overflowed_cos0 (i = %d, rnd = %s):\n"
+                      "  The overflow flag is not set.\n",
+                      i, mpfr_print_rnd_mode (rnd));
+              err = 1;
+            }
+          if (rnd == GMP_RNDZ || rnd == GMP_RNDD)
+            {
+              if (inex >= 0)
+                {
+                  printf ("Error in overflowed_cos0 (i = %d, rnd = %s):\n"
+                          "  The inexact value must be negative.\n",
+                          i, mpfr_print_rnd_mode (rnd));
+                  err = 1;
+                }
+              if (! mpfr_equal_p (x, y))
+                {
+                  printf ("Error in overflowed_cos0 (i = %d, rnd = %s):\n"
+                          "  Got ", i, mpfr_print_rnd_mode (rnd));
+                  mpfr_print_binary (x);
+                  printf (" instead of 0.11111111E%d.\n", emax);
+                  err = 1;
+                }
+            }
+          else
+            {
+              if (inex <= 0)
+                {
+                  printf ("Error in overflowed_cos0 (i = %d, rnd = %s):\n"
+                          "  The inexact value must be positive.\n",
+                          i, mpfr_print_rnd_mode (rnd));
+                  err = 1;
+                }
+              if (! (mpfr_inf_p (x) && MPFR_SIGN (x) > 0))
+                {
+                  printf ("Error in overflowed_cos0 (i = %d, rnd = %s):\n"
+                          "  Got ", i, mpfr_print_rnd_mode (rnd));
+                  mpfr_print_binary (x);
+                  printf (" instead of +Inf.\n");
+                  err = 1;
+                }
+            }
+        }
+      set_emax (old_emax);
+    }
+
+  if (err)
+    exit (1);
+  mpfr_clear (x);
+  mpfr_clear (y);
+}
+
 int
 main (int argc, char *argv[])
 {
   mpfr_t x, y;
+  int inex;
 
   tests_start_mpfr ();
 
@@ -242,6 +327,12 @@ main (int argc, char *argv[])
       exit (1);
     }
 
+  mpfr_set_prec (x, 3);
+  mpfr_set_prec (y, 3);
+  mpfr_set_str_binary (x, "0.110E60");
+  inex = mpfr_cos (y, x, GMP_RNDD);
+  MPFR_ASSERTN(inex < 0);
+
   /* worst case from PhD thesis of Vincent Lefe`vre: x=8980155785351021/2^54 */
   check53 ("4.984987858808754279e-1", "8.783012931285841817e-1", GMP_RNDN);
   check53 ("4.984987858808754279e-1", "8.783012931285840707e-1", GMP_RNDD);
@@ -254,11 +345,23 @@ main (int argc, char *argv[])
 
   check53 ("1.00591265847407274059", "0.53531755997839769456",  GMP_RNDN);
 
+  overflowed_cos0 ();
   test_generic (2, 100, 15);
 
+  /* check inexact flag */
+  mpfr_set_prec (x, 3);
+  mpfr_set_prec (y, 13);
+  mpfr_set_str_binary (x, "-0.100E196");
+  inex = mpfr_cos (y, x, GMP_RNDU);
+  mpfr_set_prec (x, 13);
+  mpfr_set_str_binary (x, "0.1111111100101");
+  MPFR_ASSERTN (inex > 0 && mpfr_equal_p (x, y));
+
   mpfr_clear (x);
   mpfr_clear (y);
 
+  data_check ("data/cos", mpfr_cos, "mpfr_cos");
+
   tests_end_mpfr ();
   return 0;
 }
