@@ -1,6 +1,6 @@
-/* Test file for mpfr_erf.
+/* Test file for mpfr_erf and mpfr_erfc.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 Contributed by Ludovic Meunier and Paul Zimmermann.
 
 This file is part of the MPFR Library.
@@ -364,7 +364,7 @@ special_erfc (void)
 {
   mpfr_t x, y;
 
-  mpfr_inits (x, y, NULL);
+  mpfr_inits (x, y, (void *) 0);
 
   /* erfc (NaN) = NaN */
   mpfr_set_nan (x);
@@ -405,7 +405,7 @@ special_erfc (void)
       exit (1);
     }
 
-  mpfr_clears (x, y, NULL);
+  mpfr_clears (x, y, (void *) 0);
 }
 
 static void
@@ -483,6 +483,33 @@ large_arg (void)
       exit (1);
     }
 
+  /* Reported by Christopher Creutzig on 2007-07-10. */
+  mpfr_set_prec (x, 53);
+  mpfr_set_prec (y, 53);
+  mpfr_set_si_2exp (x, 54563, -1, GMP_RNDN);
+  mpfr_erfc (y, x, GMP_RNDZ);
+  mpfr_set_ui (x, 0, GMP_RNDN);
+  if (! mpfr_equal_p (y, x))
+    {
+      printf ("mpfr_erfc failed for x=27281.5, prec=53 (6)\n");
+      printf ("expected "); mpfr_dump (x);
+      printf ("got      "); mpfr_dump (y);
+      exit (1);
+    }
+
+  /* same test with rounding away from zero */
+  mpfr_set_si_2exp (x, 54563, -1, GMP_RNDN);
+  mpfr_erfc (y, x, GMP_RNDU);
+  mpfr_set_ui (x, 0, GMP_RNDN);
+  mpfr_nextabove (x);
+  if (! mpfr_equal_p (y, x))
+    {
+      printf ("mpfr_erfc failed for x=27281.5, prec=53 (7)\n");
+      printf ("expected "); mpfr_dump (x);
+      printf ("got      "); mpfr_dump (y);
+      exit (1);
+    }
+
   mpfr_clear (x);
   mpfr_clear (y);
 }
@@ -491,6 +518,7 @@ static void
 test_erfc (void)
 {
   mpfr_t x, y, z;
+  int inex;
 
   mpfr_inits2 (40, x, y, z, (void *) 0);
 
@@ -508,6 +536,12 @@ test_erfc (void)
       exit (1);
     }
 
+  /* slowness detected by Kevin Rauch on 26 Oct 2007 */
+  mpfr_set_prec (x, 128);
+  mpfr_set_si (x, -256, GMP_RNDN);
+  inex = mpfr_erfc (x, x, GMP_RNDN);
+  MPFR_ASSERTN(inex > 0 && mpfr_cmp_ui (x, 2) == 0);
+
   mpfr_clears (x, y, z, (void *) 0);
 }
 
@@ -524,6 +558,9 @@ main (int argc, char *argv[])
   test_generic_erf (2, 100, 15);
   test_generic_erfc (2, 100, 15);
 
+  data_check ("data/erf",  mpfr_erf,  "mpfr_erf");
+  data_check ("data/erfc", mpfr_erfc, "mpfr_erfc");
+
   tests_end_mpfr ();
   return 0;
 }
