@@ -1,6 +1,7 @@
 /* Test file for mpfr_exp.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -53,7 +54,7 @@ check3 (const char *op, mp_rnd_t rnd, const char *res)
 {
   mpfr_t x, y;
 
-  mpfr_inits2 (53, x, y, NULL);
+  mpfr_inits2 (53, x, y, (void *) 0);
   /* y negative. If we forget to set the sign in mpfr_exp, we'll see it. */
   mpfr_set_si (y, -1, GMP_RNDN);
   mpfr_set_str1 (x, op);
@@ -67,7 +68,7 @@ check3 (const char *op, mp_rnd_t rnd, const char *res)
       putchar('\n');
       exit (1);
     }
-  mpfr_clears (x, y, NULL);
+  mpfr_clears (x, y, (void *) 0);
 }
 
 /* expx is the value of exp(X) rounded towards -infinity */
@@ -76,7 +77,7 @@ check_worst_case (const char *Xs, const char *expxs)
 {
   mpfr_t x, y;
 
-  mpfr_inits2(53, x, y, NULL);
+  mpfr_inits2 (53, x, y, (void *) 0);
   mpfr_set_str1(x, Xs);
   test_exp(y, x, GMP_RNDD);
   if (mpfr_cmp_str1 (y, expxs))
@@ -92,7 +93,7 @@ check_worst_case (const char *Xs, const char *expxs)
       printf ("exp(x) rounded towards +infinity is wrong\n");
       exit(1);
     }
-  mpfr_clears(x,y,NULL);
+  mpfr_clears (x, y, (void *) 0);
 }
 
 /* worst cases communicated by Jean-Michel Muller and Vincent Lefevre */
@@ -178,7 +179,7 @@ compare_exp2_exp3 (mp_prec_t p0, mp_prec_t p1)
                   mpfr_print_rnd_mode (rnd));
           mpfr_print_binary (x);
           puts ("");
-          printf ("mpfr_exp_2 gives  ");
+          printf ("mpfr_exp_2 gives ");
           mpfr_print_binary (y);
           puts ("");
           printf ("mpfr_exp_3 gives ");
@@ -220,6 +221,9 @@ check_special ()
   mpfr_t x, y, z;
   mp_exp_t emin, emax;
 
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
+
   mpfr_init (x);
   mpfr_init (y);
   mpfr_init (z);
@@ -289,36 +293,36 @@ check_special ()
       exit (1);
     }
   /* Check overflow. Corner case of mpfr_exp_3 */
-  if (MPFR_PREC_MAX > MPFR_EXP_THRESHOLD+10) {
-    mpfr_set_prec (x, MPFR_EXP_THRESHOLD+10);
-    mpfr_set_str (x,
-     "0.1011000101110010000101111111010100001100000001110001100111001101E30",
-                  2, GMP_RNDN);
-    mpfr_clear_overflow ();
-    mpfr_exp (x, x, GMP_RNDD);
-    if (!mpfr_overflow_p ())
-      {
-        printf ("Wrong overflow detection in mpfr_exp_3\n");
-        mpfr_dump (x);
-        exit (1);
-      }
-    /* Check underflow. Corner case of mpfr_exp_3 */
-    mpfr_set_str (x,
-    "-0.1011000101110010000101111111011111010001110011110111100110101100E30",
-                  2, GMP_RNDN);
-    mpfr_clear_underflow ();
-    mpfr_exp (x, x, GMP_RNDN);
-    if (!mpfr_underflow_p ())
-      {
-        printf ("Wrong underflow detection in mpfr_exp_3\n");
-        mpfr_dump (x);
-        exit (1);
-      }
-    mpfr_set_prec (x, 53);
-  }
+  if (MPFR_PREC_MAX > MPFR_EXP_THRESHOLD + 10)
+    {
+      mpfr_set_prec (x, MPFR_EXP_THRESHOLD + 10);
+      mpfr_set_str (x,
+       "0.1011000101110010000101111111010100001100000001110001100111001101E30",
+                    2, GMP_RNDN);
+      mpfr_clear_overflow ();
+      mpfr_exp (x, x, GMP_RNDD);
+      if (!mpfr_overflow_p ())
+        {
+          printf ("Wrong overflow detection in mpfr_exp_3\n");
+          mpfr_dump (x);
+          exit (1);
+        }
+      /* Check underflow. Corner case of mpfr_exp_3 */
+      mpfr_set_str (x,
+      "-0.1011000101110010000101111111011111010001110011110111100110101100E30",
+                    2, GMP_RNDN);
+      mpfr_clear_underflow ();
+      mpfr_exp (x, x, GMP_RNDN);
+      if (!mpfr_underflow_p ())
+        {
+          printf ("Wrong underflow detection in mpfr_exp_3\n");
+          mpfr_dump (x);
+          exit (1);
+        }
+      mpfr_set_prec (x, 53);
+    }
 
   /* check overflow */
-  emax = mpfr_get_emax ();
   set_emax (10);
   mpfr_set_ui (x, 7, GMP_RNDN);
   test_exp (y, x, GMP_RNDN);
@@ -330,7 +334,6 @@ check_special ()
   set_emax (emax);
 
   /* check underflow */
-  emin = mpfr_get_emin ();
   set_emin (-10);
   mpfr_set_si (x, -9, GMP_RNDN);
   test_exp (y, x, GMP_RNDN);
@@ -418,8 +421,8 @@ check_special ()
       exit (1);
     }
 
-  set_emin (MPFR_EMIN_MIN);
-  set_emax (MPFR_EMAX_MAX);
+  set_emin (emin);
+  set_emax (emax);
 
   /* Check for overflow producing a segfault with HUGE exponent */
   mpfr_set_ui  (x, 3, GMP_RNDN);
@@ -503,6 +506,84 @@ check_exp10(void)
   mpfr_clear (x);
 }
 
+static void
+overflowed_exp0 (void)
+{
+  mpfr_t x, y;
+  int emax, i, inex, rnd, err = 0;
+  mp_exp_t old_emax;
+
+  old_emax = mpfr_get_emax ();
+
+  mpfr_init2 (x, 8);
+  mpfr_init2 (y, 8);
+
+  for (emax = -1; emax <= 0; emax++)
+    {
+      mpfr_set_ui_2exp (y, 1, emax, GMP_RNDN);
+      mpfr_nextbelow (y);
+      set_emax (emax);  /* 1 is not representable. */
+      /* and if emax < 0, 1 - eps is not representable either. */
+      for (i = -1; i <= 1; i++)
+        RND_LOOP (rnd)
+        {
+          mpfr_set_si_2exp (x, i, -512 * ABS (i), GMP_RNDN);
+          mpfr_clear_flags ();
+          inex = mpfr_exp (x, x, rnd);
+          if ((i >= 0 || emax < 0 || rnd == GMP_RNDN || rnd == GMP_RNDU) &&
+              ! mpfr_overflow_p ())
+            {
+              printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                      "  The overflow flag is not set.\n",
+                      i, mpfr_print_rnd_mode (rnd));
+              err = 1;
+            }
+          if (rnd == GMP_RNDZ || rnd == GMP_RNDD)
+            {
+              if (inex >= 0)
+                {
+                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                          "  The inexact value must be negative.\n",
+                          i, mpfr_print_rnd_mode (rnd));
+                  err = 1;
+                }
+              if (! mpfr_equal_p (x, y))
+                {
+                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                          "  Got ", i, mpfr_print_rnd_mode (rnd));
+                  mpfr_print_binary (x);
+                  printf (" instead of 0.11111111E%d.\n", emax);
+                  err = 1;
+                }
+            }
+          else
+            {
+              if (inex <= 0)
+                {
+                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                          "  The inexact value must be positive.\n",
+                          i, mpfr_print_rnd_mode (rnd));
+                  err = 1;
+                }
+              if (! (mpfr_inf_p (x) && MPFR_SIGN (x) > 0))
+                {
+                  printf ("Error in overflowed_exp0 (i = %d, rnd = %s):\n"
+                          "  Got ", i, mpfr_print_rnd_mode (rnd));
+                  mpfr_print_binary (x);
+                  printf (" instead of +Inf.\n");
+                  err = 1;
+                }
+            }
+        }
+      set_emax (old_emax);
+    }
+
+  if (err)
+    exit (1);
+  mpfr_clear (x);
+  mpfr_clear (y);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -560,6 +641,10 @@ main (int argc, char *argv[])
   check3("6.00812634798592370977e-01", GMP_RNDN, "1.823600119339019443");
   check_exp10 ();
 
+  overflowed_exp0 ();
+
+  data_check ("data/exp", mpfr_exp, "mpfr_exp");
+
   tests_end_mpfr ();
   return 0;
 }
