@@ -1,7 +1,7 @@
 /* Test file for mpfr_exp2.
 
-Copyright 2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
-Adapted from tarctan.c.
+Copyright 2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -118,12 +118,14 @@ static void
 exp_range (void)
 {
   mpfr_t x;
+  mp_exp_t emin;
 
+  emin = mpfr_get_emin ();
   set_emin (3);
   mpfr_init2 (x, 8);
   mpfr_set_ui (x, 5, GMP_RNDN);
   mpfr_exp2 (x, x, GMP_RNDN);
-  set_emin (MPFR_EMIN_MIN);
+  set_emin (emin);
   if (mpfr_nan_p (x) || mpfr_cmp_ui (x, 32) != 0)
     {
       printf ("Error in mpfr_exp2 for x = 5, with emin = 3\n");
@@ -135,6 +137,84 @@ exp_range (void)
   mpfr_clear (x);
 }
 
+static void
+overflowed_exp2_0 (void)
+{
+  mpfr_t x, y;
+  int emax, i, inex, rnd, err = 0;
+  mp_exp_t old_emax;
+
+  old_emax = mpfr_get_emax ();
+
+  mpfr_init2 (x, 8);
+  mpfr_init2 (y, 8);
+
+  for (emax = -1; emax <= 0; emax++)
+    {
+      mpfr_set_ui_2exp (y, 1, emax, GMP_RNDN);
+      mpfr_nextbelow (y);
+      set_emax (emax);  /* 1 is not representable. */
+      /* and if emax < 0, 1 - eps is not representable either. */
+      for (i = -1; i <= 1; i++)
+        RND_LOOP (rnd)
+          {
+            mpfr_set_si_2exp (x, i, -512 * ABS (i), GMP_RNDN);
+            mpfr_clear_flags ();
+            inex = mpfr_exp2 (x, x, rnd);
+            if ((i >= 0 || emax < 0 || rnd == GMP_RNDN || rnd == GMP_RNDU) &&
+                ! mpfr_overflow_p ())
+              {
+                printf ("Error in overflowed_exp2_0 (i = %d, rnd = %s):\n"
+                        "  The overflow flag is not set.\n",
+                        i, mpfr_print_rnd_mode (rnd));
+                err = 1;
+              }
+            if (rnd == GMP_RNDZ || rnd == GMP_RNDD)
+              {
+                if (inex >= 0)
+                  {
+                    printf ("Error in overflowed_exp2_0 (i = %d, rnd = %s):\n"
+                            "  The inexact value must be negative.\n",
+                            i, mpfr_print_rnd_mode (rnd));
+                    err = 1;
+                  }
+                if (! mpfr_equal_p (x, y))
+                  {
+                    printf ("Error in overflowed_exp2_0 (i = %d, rnd = %s):\n"
+                            "  Got ", i, mpfr_print_rnd_mode (rnd));
+                    mpfr_print_binary (x);
+                    printf (" instead of 0.11111111E%d.\n", emax);
+                    err = 1;
+                  }
+              }
+            else
+              {
+                if (inex <= 0)
+                  {
+                    printf ("Error in overflowed_exp2_0 (i = %d, rnd = %s):\n"
+                            "  The inexact value must be positive.\n",
+                            i, mpfr_print_rnd_mode (rnd));
+                    err = 1;
+                  }
+                if (! (mpfr_inf_p (x) && MPFR_SIGN (x) > 0))
+                  {
+                    printf ("Error in overflowed_exp2_0 (i = %d, rnd = %s):\n"
+                            "  Got ", i, mpfr_print_rnd_mode (rnd));
+                    mpfr_print_binary (x);
+                    printf (" instead of +Inf.\n");
+                    err = 1;
+                  }
+              }
+          }
+      set_emax (old_emax);
+    }
+
+  if (err)
+    exit (1);
+  mpfr_clear (x);
+  mpfr_clear (y);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -152,28 +232,48 @@ main (int argc, char *argv[])
 
   mpfr_set_ui (x, 4, GMP_RNDN);
   mpfr_exp2 (y, x, GMP_RNDN);
+  if (mpfr_cmp_ui (y, 16) != 0)
+    {
+      printf ("Error for 2^4, GMP_RNDN\n");
+      exit (1);
+    }
   mpfr_exp2 (y, x, GMP_RNDD);
+  if (mpfr_cmp_ui (y, 16) != 0)
+    {
+      printf ("Error for 2^4, GMP_RNDD\n");
+      exit (1);
+    }
   mpfr_exp2 (y, x, GMP_RNDU);
   if (mpfr_cmp_ui (y, 16) != 0)
     {
-      printf ("Error for 2^4\n");
+      printf ("Error for 2^4, GMP_RNDU\n");
       exit (1);
     }
 
   mpfr_set_si (x, -4, GMP_RNDN);
   mpfr_exp2 (y, x, GMP_RNDN);
+  if (mpfr_cmp_ui_2exp (y, 1, -4) != 0)
+    {
+      printf ("Error for 2^(-4), GMP_RNDN\n");
+      exit (1);
+    }
   mpfr_exp2 (y, x, GMP_RNDD);
+  if (mpfr_cmp_ui_2exp (y, 1, -4) != 0)
+    {
+      printf ("Error for 2^(-4), GMP_RNDD\n");
+      exit (1);
+    }
   mpfr_exp2 (y, x, GMP_RNDU);
   if (mpfr_cmp_ui_2exp (y, 1, -4) != 0)
     {
-      printf ("Error for 2^(-4)\n");
+      printf ("Error for 2^(-4), GMP_RNDU\n");
       exit (1);
     }
 
   mpfr_set_prec (x, 53);
   mpfr_set_prec (y, 53);
   mpfr_set_str (x, /*-1683977482443233.0 / 2199023255552.0*/
-                "-7.6578429909351734750089235603809357e2", 10,GMP_RNDN);
+                "-7.6578429909351734750089235603809357e2", 10, GMP_RNDN);
   mpfr_exp2 (y, x, GMP_RNDN);
   if (mpfr_cmp_str1 (y, "2.991959870867646566478e-231"))
     {
@@ -211,7 +311,6 @@ main (int argc, char *argv[])
   /* restore emax */
   set_emax (emax);
 
-
   MPFR_SET_INF(x);
   MPFR_SET_POS(x);
   mpfr_exp2 (y, x, GMP_RNDN);
@@ -253,11 +352,26 @@ main (int argc, char *argv[])
         }
     }
 
+  mpfr_set_prec (x, 2);
+  mpfr_set_prec (y, 2);
+  mpfr_set_str_binary (x, "-1.0E-26");
+  mpfr_exp2 (y, x, GMP_RNDD);
+  mpfr_set_str_binary (x, "1.1E-1");
+  if (mpfr_cmp (x, y))
+    {
+      printf ("Error for exp(-2^(-26)) for prec=2\n");
+      exit (1);
+    }
+
   test_generic (2, 100, 100);
 
   mpfr_clear (x);
   mpfr_clear (y);
 
+  overflowed_exp2_0 ();
+
+  data_check ("data/exp2", mpfr_exp2, "mpfr_exp2");
+
   tests_end_mpfr ();
   return 0;
 }
