@@ -1,6 +1,7 @@
 /* Test file for mpfr_get_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -1025,32 +1026,55 @@ check_special (int b, mp_prec_t p)
 }
 
 static void
-check_bug_base2k(void)
+check_bug_base2k (void)
 {
   /*
    * -2.63b22b55697e800000000000@130
    * +-0.1001100011101100100010101101010101011010010111111010000000000000000000000000+00000000000000000000001E522
   */
-  mpfr_t xx,yy,zz;
+  mpfr_t xx, yy, zz;
   char *s;
   mp_exp_t e;
 
-  mpfr_init2(xx,107);
-  mpfr_init2(yy,79);
-  mpfr_init2(zz,99);
+  mpfr_init2 (xx, 107);
+  mpfr_init2 (yy, 79);
+  mpfr_init2 (zz, 99);
 
-  mpfr_set_str(xx, "-1.90e8c3e525d7c0000000000000@-18", 16, GMP_RNDN);
-  mpfr_set_str(yy, "-2.63b22b55697e8000000@130", 16, GMP_RNDN);
-  mpfr_add(zz, xx, yy, GMP_RNDD);
-  s = mpfr_get_str(NULL, &e, 16, 0, zz, GMP_RNDN);
-  if (strcmp(s, "-263b22b55697e8000000000008"))
+  mpfr_set_str (xx, "-1.90e8c3e525d7c0000000000000@-18", 16, GMP_RNDN);
+  mpfr_set_str (yy, "-2.63b22b55697e8000000@130", 16, GMP_RNDN);
+  mpfr_add (zz, xx, yy, GMP_RNDD);
+  s = mpfr_get_str (NULL, &e, 16, 0, zz, GMP_RNDN);
+  if (strcmp (s, "-263b22b55697e8000000000008"))
     {
-      printf(
-"Error for get_str base 16\nGot %s expected -263b22b55697e8000000000008\n", s);
-      exit(1);
+      printf ("Error for get_str base 16\n"
+              "Got %s expected -263b22b55697e8000000000008\n", s);
+      exit (1);
     }
   mpfr_free_str (s);
-  mpfr_clears(xx,yy,zz,NULL);
+  mpfr_clears (xx, yy, zz, (void *) 0);
+}
+
+static void
+check_reduced_exprange (void)
+{
+  mpfr_t x;
+  char *s;
+  mp_exp_t emax, e;
+
+  emax = mpfr_get_emax ();
+  mpfr_init2 (x, 8);
+  mpfr_set_str (x, "0.11111111E0", 2, GMP_RNDN);
+  set_emax (0);
+  s = mpfr_get_str (NULL, &e, 16, 0, x, GMP_RNDN);
+  set_emax (emax);
+  if (strcmp (s, "ff0"))
+    {
+      printf ("Error for mpfr_get_str on 0.11111111E0 in base 16:\n"
+              "Got \"%s\" instead of \"ff0\".\n", s);
+      exit (1);
+    }
+  mpfr_free_str (s);
+  mpfr_clear (x);
 }
 
 #define ITER 1000
@@ -1071,6 +1095,7 @@ main (int argc, char *argv[])
 
   check_small ();
 
+  check_special (2, 2);
   for (i = 0; i < ITER; i++)
     {
       p = 2 + (randlimb () % (MAX_DIGITS - 1));
@@ -1111,7 +1136,9 @@ main (int argc, char *argv[])
   check3 (6.7274500420134077e-87, GMP_RNDU, "67275");
   check3 (6.7274500420134077e-87, GMP_RNDD, "67274");
 
-  check_bug_base2k();
+  check_bug_base2k ();
+  check_reduced_exprange ();
+
   tests_end_mpfr ();
   return 0;
 }
