@@ -1,7 +1,7 @@
 /* Test file for mpfr_log2.
 
-Copyright 2001, 2002, 2004, 2006 Free Software Foundation, Inc.
-Adapted from tsinh.c.
+Copyright 2001, 2002, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -32,36 +32,37 @@ static void
 special (void)
 {
   mpfr_t x;
+  int inex;
 
   mpfr_init (x);
 
   mpfr_set_nan (x);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_nan_p (x));
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
   mpfr_set_inf (x, -1);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_nan_p (x));
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
   mpfr_set_inf (x, 1);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_inf_p (x) && mpfr_sgn (x) > 0);
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) > 0 && inex == 0);
 
   mpfr_set_ui (x, 0, GMP_RNDN);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_inf_p (x) && mpfr_sgn (x) < 0);
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
   mpfr_set_ui (x, 0, GMP_RNDN);
   mpfr_neg (x, x, GMP_RNDN);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_inf_p (x) && mpfr_sgn (x) < 0);
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
 
   mpfr_set_si (x, -1, GMP_RNDN);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_nan_p (x));
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
   mpfr_set_si (x, 1, GMP_RNDN);
-  mpfr_log2 (x, x, GMP_RNDN);
-  MPFR_ASSERTN(mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS(x));
+  inex = mpfr_log2 (x, x, GMP_RNDN);
+  MPFR_ASSERTN (mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS(x) && inex == 0);
 
   mpfr_clear (x);
 }
@@ -75,6 +76,8 @@ main (int argc, char *argv[])
 
   test_generic (2, 100, 30);
 
+  data_check ("data/log2", mpfr_log2, "mpfr_log2");
+
   tests_end_mpfr ();
   return 0;
 }
