@@ -1,6 +1,7 @@
 /* Test file for mpfr_{mul,div}_2{ui,si}.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -53,7 +54,7 @@ main (int argc, char *argv[])
 
   tests_start_mpfr ();
 
-  mpfr_inits2 (53, w, z, NULL);
+  mpfr_inits2 (53, w, z, (void *) 0);
 
   for (i = 0; i < 3; i++)
     {
@@ -131,7 +132,7 @@ main (int argc, char *argv[])
         }
     }
 
-  mpfr_clears (w, z, NULL);
+  mpfr_clears (w, z, (void *) 0);
 
   tests_end_mpfr ();
   return 0;
