@@ -1,6 +1,7 @@
 /* Test file for the various mpfr_random fonctions.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -32,12 +33,15 @@ test_random (long nbtests, mp_prec_t prec, int verbose)
   int *tab, size_tab, k;
   double d, av = 0, var = 0, chi2 = 0, th;
 
-  mpfr_init2(x, prec);
-
   size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) malloc (size_tab * sizeof(int));
-  for (k = 0; k < size_tab; ++k)
-    tab[k] = 0;
+  tab = (int *) calloc (size_tab, sizeof(int));
+  if (tab == NULL)
+    {
+      fprintf (stderr, "trandom: can't allocate memory in test_random\n");
+      exit (1);
+    }
+
+  mpfr_init2(x, prec);
 
   for (k = 0; k < nbtests; k++)
     {
@@ -54,7 +58,7 @@ test_random (long nbtests, mp_prec_t prec, int verbose)
     }
 
   av /= nbtests;
-  var = (var /nbtests) - av*av;
+  var = (var / nbtests) - av*av;
 
   th = (double) nbtests / size_tab;
 
@@ -86,15 +90,18 @@ test_random2 (long nbtests, mp_prec_t prec, int verbose)
   int *tab, size_tab, k, sh, xn;
   double d, av = 0, var = 0, chi2 = 0, th;
 
+  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
+  tab = (int *) calloc (size_tab, sizeof(int));
+  if (tab == NULL)
+    {
+      fprintf (stderr, "trandom: can't allocate memory in test_random2\n");
+      exit (1);
+    }
+
   mpfr_init2 (x, prec);
   xn = 1 + (prec - 1) / mp_bits_per_limb;
   sh = xn * mp_bits_per_limb - prec;
 
-  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) malloc (size_tab * sizeof(int));
-  for (k = 0; k < size_tab; ++k)
-    tab[k] = 0;
-
   for (k = 0; k < nbtests; k++)
     {
       mpfr_random2 (x, xn, 0);
@@ -105,6 +112,15 @@ test_random2 (long nbtests, mp_prec_t prec, int verbose)
           mpfr_print_binary (x); puts ("");
           exit (1);
         }
+
+      /* check that the number is normalized */
+      if (! (MPFR_MANT(x)[MPFR_LIMB_SIZE(x) - 1] >> (BITS_PER_MP_LIMB - 1)))
+        {
+          printf ("Error: mpfr_random2() returns unnormalized numbers:\n");
+          mpfr_print_binary (x); puts ("");
+          exit (1);
+        }
+
       /* check that exponent is in correct range */
       if (mpfr_get_exp (x) != 0)
         {
@@ -137,7 +153,7 @@ test_random2 (long nbtests, mp_prec_t prec, int verbose)
     }
 
   av /= nbtests;
-  var = (var /nbtests) - av*av;
+  var = (var / nbtests) - av*av;
 
   th = (double)nbtests / size_tab;
   printf("Average = %.5f\nVariance = %.5f\n", av, var);
@@ -167,15 +183,18 @@ test_urandomb (long nbtests, mp_prec_t prec, int verbose)
   double d, av = 0, var = 0, chi2 = 0, th;
   mp_exp_t emin;
 
+  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
+  tab = (int *) calloc (size_tab, sizeof(int));
+  if (tab == NULL)
+    {
+      fprintf (stderr, "trandom: can't allocate memory in test_urandomb\n");
+      exit (1);
+    }
+
   mpfr_init2 (x, prec);
   xn = 1 + (prec - 1) / mp_bits_per_limb;
   sh = xn * mp_bits_per_limb - prec;
 
-  size_tab = (nbtests >= 1000 ? nbtests / 50 : 20);
-  tab = (int *) malloc (size_tab * sizeof(int));
-  for (k = 0; k < size_tab; ++k)
-    tab[k] = 0;
-
   gmp_randinit (state, GMP_RAND_ALG_LC, 128);
   gmp_randseed_ui (state, time(NULL));
 
@@ -215,7 +234,7 @@ test_urandomb (long nbtests, mp_prec_t prec, int verbose)
     }
 
   av /= nbtests;
-  var = (var /nbtests) - av*av;
+  var = (var / nbtests) - av * av;
 
   th = (double)nbtests / size_tab;
   printf("Average = %.5f\nVariance = %.5f\n", av, var);
