@@ -1,6 +1,7 @@
 /* Test file for mpfr_set_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -92,11 +93,12 @@ main (int argc, char *argv[])
 
   tests_start_mpfr ();
 
-  if (argc>=2) /* tset_str <string> <prec> */
+  if (argc >= 2) /* tset_str <string> [<prec>] [<base>] */
     {
-      prec = (argc>=3) ? atoi(argv[2]) : 53;
+      prec = (argc >= 3) ? atoi (argv[2]) : 53;
+      base = (argc >= 4) ? atoi (argv[3]) : 2;
       mpfr_init2 (x, prec);
-      mpfr_set_str_binary (x, argv[1]);
+      mpfr_set_str (x, argv[1], base, GMP_RNDN);
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN);
       puts ("");
       mpfr_clear (x);
@@ -805,9 +807,6 @@ main (int argc, char *argv[])
     }
 
   /* check invalid input */
-  mpfr_set_ui (x, 1, GMP_RNDN);
-  ret = mpfr_set_str (x, "1", 37, GMP_RNDN);
-  MPFR_ASSERTN (ret == -1);
   ret = mpfr_set_str (x, "1E10toto", 10, GMP_RNDN);
   MPFR_ASSERTN (ret == -1);
   ret = mpfr_set_str (x, "1p10toto", 16, GMP_RNDN);
