@@ -1,6 +1,7 @@
 /* Test file for mpfr_sqr.
 
-Copyright 2004, 2005, 2006 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -27,6 +28,9 @@ MA 02110-1301, USA. */
 void check_special (void);
 void check_random (mpfr_prec_t p);
 
+#define TEST_FUNCTION mpfr_sqr
+#include "tgeneric.c"
+
 int main(void)
 {
   mpfr_prec_t p;
@@ -37,6 +41,8 @@ int main(void)
   for(p = 2 ; p < 200 ; p++)
     check_random (p);
 
+  test_generic (2, 200, 15);
+
   tests_end_mpfr ();
   return 0;
 }
@@ -75,7 +81,7 @@ void check_random(mpfr_prec_t p)
   int r;
   int i, inexact1, inexact2;
 
-  mpfr_inits2(p, x, y, z, NULL);
+  mpfr_inits2 (p, x, y, z, (void *) 0);
   for(i = 0 ; i < 500 ; i++)
     {
       mpfr_random (x);
@@ -90,7 +96,7 @@ void check_random(mpfr_prec_t p)
               error2 ((mp_rnd_t) r,p,x,y,inexact1,inexact2);
           }
     }
-  mpfr_clears(x,y,z,NULL);
+  mpfr_clears (x, y, z, (void *) 0);
 }
 
 void check_special(void)
