@@ -1,7 +1,7 @@
 /* Test file for mpfr_tanh.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
-Adapted from tarctan.c.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -60,6 +60,10 @@ special_overflow (void)
 {
   mpfr_t x, y;
   int i;
+  mp_exp_t emin, emax;
+
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
 
   mpfr_clear_overflow ();
   set_emin (-125);
@@ -87,8 +91,8 @@ special_overflow (void)
     }
   MPFR_ASSERTN (!mpfr_overflow_p ());
 
-  set_emin (MPFR_EMIN_MIN);
-  set_emax (MPFR_EMAX_MAX);
+  set_emin (emin);
+  set_emax (emax);
 
   mpfr_set_str_binary (x, "0.1E1000000000");
   i = mpfr_tanh (y, x, GMP_RNDN);
@@ -123,6 +127,8 @@ main (int argc, char *argv[])
 
   test_generic (2, 100, 100);
 
+  data_check ("data/tanh", mpfr_tanh, "mpfr_tanh");
+
   tests_end_mpfr ();
   return 0;
 }
